From eb20104170613c345d494fce80f5711d5d9c9cc3 Mon Sep 17 00:00:00 2001
From: Hugo Hallqvist <hugo@dokad.se>
Date: Sat, 1 Nov 2008 01:26:38 +0100
Subject: [PATCH] Added simple wildcard matching to status cgi.

This patch enables wildcard matching of hostnames when calling status
cgi from the menus "Show host" editbox. If an asterisk (*) is found in
the hostname, the regex matching kicks in. First we create a regex
string by replacing all * with .*. This string is compiled and used
later on to match the hostname of the host.
---
 cgi/status.c |   58 +++++++++++++++++++++++++++++++++++++++++-----------------
 1 files changed, 41 insertions(+), 17 deletions(-)

diff --git a/cgi/status.c b/cgi/status.c
index a869d56..5f99591 100644
--- a/cgi/status.c
+++ b/cgi/status.c
@@ -138,6 +138,7 @@ time_t current_time;
 
 char alert_message[MAX_MESSAGE_BUFFER];
 char *host_name=NULL;
+char *host_filter=NULL;
 char *hostgroup_name=NULL;
 char *servicegroup_name=NULL;
 char *service_filter=NULL;
@@ -242,28 +243,46 @@ int main(void){
 
 	/* if a navbar search was performed, find the host by name, address or partial name */
 	if(navbar_search==TRUE){
-		if((temp_host=find_host(host_name))==NULL){
-			for(temp_host=host_list;temp_host!=NULL;temp_host=temp_host->next){
-				if(is_authorized_for_host(temp_host,&current_authdata)==FALSE)
-					continue;
-				if(!strcmp(host_name,temp_host->address)){
-					free(host_name);
-					host_name=strdup(temp_host->name);
-					break;
-			                }
-		                }
-			if(temp_host==NULL){
+		if(host_name!=NULL && NULL!=strstr(host_name, "*")){
+			/* allocate for 3 extra chars, ^, $ and \0 */
+			host_filter = malloc(sizeof(char) * (strlen(host_name) * 2 + 3));
+			int regex_i=1,i=0;
+			int len=strlen(host_name);
+			for (i=0;i<len;i++,regex_i++) {
+				if(host_name[i]=='*') {
+					host_filter[regex_i++]='.';
+					host_filter[regex_i]='*';
+					}
+				else
+					host_filter[regex_i]=host_name[i];
+				}
+			host_filter[0]='^';
+			host_filter[regex_i++]='$';
+			host_filter[regex_i]='\0';
+			}
+		else{
+			if((temp_host=find_host(host_name))==NULL){
 				for(temp_host=host_list;temp_host!=NULL;temp_host=temp_host->next){
 					if(is_authorized_for_host(temp_host,&current_authdata)==FALSE)
 						continue;
-					if((strstr(temp_host->name,host_name)==temp_host->name) || !strncasecmp(temp_host->name,host_name,strlen(host_name))){
+					if(!strcmp(host_name,temp_host->address)){
 						free(host_name);
 						host_name=strdup(temp_host->name);
 						break;
-			                        }
-		                        }
-			        }
-			}
+						}
+					}
+				if(temp_host==NULL){
+					for(temp_host=host_list;temp_host!=NULL;temp_host=temp_host->next){
+						if(is_authorized_for_host(temp_host,&current_authdata)==FALSE)
+							continue;
+						if((strstr(temp_host->name,host_name)==temp_host->name) || !strncasecmp(temp_host->name,host_name,strlen(host_name))){
+							free(host_name);
+							host_name=strdup(temp_host->name);
+							break;
+							}
+						}
+					}
+				}
 			/* last effort, search hostgroups then servicegroups */
 			if(temp_host==NULL){
 				if((temp_hostgroup=find_hostgroup(host_name))!=NULL){
@@ -277,6 +296,7 @@ int main(void){
 					show_all_servicegroups=FALSE;
 					free(host_name);
 					servicegroup_name=strdup(temp_servicegroup->group_name);
+					}
 				}
 			}
 	        }
@@ -1197,7 +1217,7 @@ void show_host_status_totals(void){
 
 /* display a detailed listing of the status of all services... */
 void show_service_detail(void){
-	regex_t preg;
+	regex_t preg, preg_hostname;
 	time_t t;
 	char date_time[MAX_DATETIME_LENGTH];
 	char state_duration[48];
@@ -1371,6 +1391,8 @@ void show_service_detail(void){
 
 	if(service_filter!=NULL)
 		regcomp(&preg,service_filter,0);
+	if(host_filter!=NULL)
+		regcomp(&preg_hostname,host_filter,REG_ICASE);
 
 	temp_hostgroup=find_hostgroup(hostgroup_name);
 	temp_servicegroup=find_servicegroup(servicegroup_name);
@@ -1445,6 +1467,8 @@ void show_service_detail(void){
 		if(display_type==DISPLAY_HOSTS){
 			if(show_all_hosts==TRUE)
 				show_service=TRUE;
+			else if(host_filter!=NULL && 0==regexec(&preg_hostname,temp_status->host_name,0,NULL,0))
+				show_service=TRUE;
 			else if(!strcmp(host_name,temp_status->host_name))
 				show_service=TRUE;
 		        }
-- 
1.5.4.3

